/**
 * Javier Abelln. 6 Abril 2006
 * 
 * Librera grfica
 */

package chuidiang.graficos;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

/**
 * Contiene mtodos de dibujado en unidades de usuario.
 */
public interface InterfaceEscalaGrafica
{
	public InterfaceZoomable getZoomable ();
    /**
     * Extremos del grfico en coordenadas de usuario.
     */
    public void tomaExtremos(double xMin, double yMin, double xMax, double yMax);

    public void pintaImagen (Point2D posicion, Image imagen);
    /**
     * Dado un punto con coordenadas x,y en pixels, devuelve un punto con las
     * coordenadas x,y del usuario correspondientes.
     * 
     * @param punto
     *            Un punto con coordenadas x,y de raton.
     * @return Un punto con coordenadas x,y de usuario.
     */
    public Point2D dameCoordenadaUsuario(Point2D punto);

    /**
     * Devuelve los extremos del grfico en coordenadas de usuario.
     */
    public Rectangle2D dameExtremos();

    /**
     * Se len pasa el alto y el ancho en pixels de la zona de dibujo.
     */
    public void tomaGraphics(Graphics g, int ancho, int alto);


    /**
     * Dibuja los puntos del x[i], y[i] del color[i]
     */
    public void pintaPuntos(Point2D[] puntos, Color[] color);

    /**
     * Dibuja los puntos x[i], y[i] de color
     */
    public void pintaPuntos(Point2D[] puntos, Color color);

    /**
     * Dibuja una lnea uniendo consecutivamente los puntos x[i], y[i] del color
     * indicado.
     */
    public void pintaPoliLinea(Point2D[] puntos, Color color);

    /**
     * Dibuja el texto que se le pasa en las coordenadas de usuario que se le
     * pasa y con el color indicado.
     */
    public void pintaTexto(Point2D posicion, String texto, Color color);

    /**
     * Dibuja un arco. La esquina es una de las esquinas del rectngulo
     * circunscrito a la circunferencia del arco. En ancho y alto son de este
     * rectngulo. El arcoInicial y arcoFinal son dnde comienza y termina el
     * arco de circunferencia, ambas en grados. El color es el color del arco.
     */
    public void pintaArco(Point2D esquina, double ancho, double alto,
            double arcoInicial, double arcoFinal, Color color);

    /**
     * Dibuja un arco relleno. La esquina es una de las esquinas del rectngulo
     * circunscrito a la circunferencia del arco. En ancho y alto son de este
     * rectngulo. El arcoInicial y arcoFinal son dnde comienza y termina el
     * arco de circunferencia, ambas en grados. El color es el color del arco.
     */
    public void pintaArcoRelleno(Point2D esquina, double ancho, double alto,
            double arcoInicial, double arcoFinal, Color color);

    /**
     * Dibuja un rectngulo relleno. Llama al mtodo correspondiente de la clase
     * Graphics.
     */
    public void pintaRectanguloRelleno(Point2D esquina, double ancho,
            double alto, Color color);

    /**
     * Dibuja un rectngulo relleno 3D. Llama al mtodo correspondiente de la
     * clase Graphics.
     */
    public void pintaRectanguloRelleno3D(Point2D esquina, double ancho,
            double alto, Color color, boolean elevado);

    /**
     * Recoge observadores a los que se avisar cuando cambien los extremos del
     * grfico (valores minimos y mximos a dibujar).
     */
    public void anhadeObservador(ObservadorEscalaGrafica unObservador);

    /**
     * Elimina un observador aadido con el mtodo anhadeObservador().
     */
    public void eliminaObservador(ObservadorEscalaGrafica unObservador);
    
    /**
     * Pinta un eje de las x en la coordenada y indicada
     * @param y Coordenada y donde se quiere el eje de las x
     * @param color Color del eje
     */
    public void pintaEjeX(double y, Color color);
    
    /**
     * Pinta un eje de las y en la coordenada x indicada
     * 
     * @param x Coordenada x donde se quiere el eje de las y
     * @param color Color del eje
     */
    public void pintaEjeY(double x, Color color);
}
