/*
 * Javier Abelln, 30 de mayo de 2004
 * ZoomPorcentaje.java
 */

package chuidiang.graficos.botones;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import chuidiang.graficos.InterfaceEscalaGrafica;
import chuidiang.graficos.InterfaceZoomable;


/**
 * Botn de zoom grfico para acercar o alejar el grfico. Se crea el botn
 * pasndole la EscalaGrfica sobre la que debe actuar y el porcentaje de
 * acercamiento o alejamiento que se quiere cada vez que se pulse el botn.
 */
public class ZoomPorcentaje extends JButton
{
    /**
	 * Serial uid
	 */
	private static final long serialVersionUID = -9044127362988998605L;

	/**
     * Constructor del botn de zoom de acercar o alejar. Se le pasa la escala
     * grfica sobre la que se quiere que actue el botn. El porcentaje de
     * acercamiento o alejamiento va entre -100 y 100, excluidos ambos valores.
     * Un porcentaje positivo hace que el grfico se acerque. Un valor negativo
     * hace que se aleje.
     */
    public ZoomPorcentaje(InterfaceEscalaGrafica escala, int porcentaje)
    {
        this.escala = escala;
        tomaPorcentaje(porcentaje);

        addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                efectuarZoom();
            }
        });
    }

    /**
     * Se le pasa el porcentaje de acercamiento del grfico cada vez que se
     * pulse el botn. El rango vlido para porcentaje es de -99 a 99. Cualquier
     * otro valor se ignora. Un porcentaje negativo har que el zoom sea de
     * alejar y uno positivo de acercar.
     */
    public void tomaPorcentaje(int porcentaje)
    {
        if ((porcentaje > -100) && (porcentaje < 100))
            this.porcentaje = porcentaje;
    }

    /**
     * Devuelve el porcentaje de acercamiento del botn
     */
    public int damePorcentaje()
    {
        return porcentaje;
    }

    /**
     * Realiza el zoom con el porcentaje interno de la clase. El porcentaje va
     * de -99 a 99. Un porcentaje positivo acerca, uno negativo aleja.
     */
    protected void efectuarZoom()
    {
    	InterfaceZoomable zoom = this.escala.getZoomable();
    	zoom.acercar(this.porcentaje);
    }

    /** Escala grfica sobre la que actua el botn de acercar. */
    protected InterfaceEscalaGrafica escala;

    /** Porcentaje de acercamiento cada vez que se pulsa el botn */
    protected int porcentaje = 10;

}
