/*
 * Javier Abelln. 25 Mayo 2004
 * 
 * Objeto grfico abstracto para pintar funciones matemticas y = f(x)
 */
package chuidiang.graficos.objetos_graficos;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import chuidiang.graficos.InterfaceEscalaGrafica;


/**
 * Clase abstracta para dibujar funciones matemticas y = f(x) sobre una clase
 * Lienzo. Debe heredarse de esta clase y redefinir el mtodo funcion(). La
 * clase se encarga del dibujado de la funcin sobre el Lienzo.
 */
public abstract class FuncionAbstracta extends AbstractObjetoGrafico
{

    /**
     * Recoge el color que se le pasa y dibujar la funcin matemtica con ese
     * color.
     */
    public FuncionAbstracta(Color color)
    {
        tomaColor(color);
    }

    /**
     * Recoge el color que se le pasa para dibujar la funcin con ese color. El
     * dibujo cambiar de color en el siguiente refresco del grfico. Si el
     * color es null, la funcin dejar de dibujarse.
     */
    public void tomaColor(Color color)
    {
        this.color = color;
    }

    /**
     * Devuelve el color con el que se est dibujando la funcin. Devuelve null
     * si previamente se le pas a la clase un color null.
     */
    public Color dameColor()
    {
        return color;
    }

    /**
     * Se le pasa el nmero de puntos a dibujar de la funcin. Se dibujarn esos
     * puntos entre la x minima y la x maxima de la escala que se pase en el
     * mtodo dibujate. Si el nmero de puntos es menor de 2, se ignora el
     * parmetro se deja el nmero de puntos anterior.
     */
    public void tomaNumeroPuntos(int numeroPuntos)
    {
        if (numeroPuntos < 2)
            return;
        this.numeroPuntos = numeroPuntos;
    }

    /**
     * Devuelve el nmero de puntos que se est utilizando para dibujar la
     * funcin.
     */
    public int dameNumeroPuntos()
    {
        return numeroPuntos;
    }

    /**
     * Dibuja la funcin matemtica sobre la escala. Obtiene la x minima y
     * maxima de la escala para hacer un bucle de 500 puntos. Calcula los 500
     * puntos de la funcin para todas esas x y dibuja segementos que unen
     * consecutivamente todos esos puntos, haciendo que la funcin quede
     * dibujada sobre una lnea.
     */
    public void dibujate(InterfaceEscalaGrafica escala)
    {
        if (color == null)
            return;

        Rectangle2D extremos = escala.dameExtremos();
        Point2D puntos[] = new Point2D[numeroPuntos];
        double x;

        for (int i = 0; i < numeroPuntos; i++)
        {
            x = extremos.getMinX() + i * extremos.getWidth() / numeroPuntos;
            puntos[i] = new Point2D.Double(x, funcion(x));
        }

        escala.pintaPoliLinea(puntos, color);
    }

    /**
     * Funcin que se va a dibujar. Las clases hijas de esta deben redefinir
     * dicha funcin, devolviendo el valor de y correspondiente a la x que se le
     * pasa.
     */
    protected abstract double funcion(double x);

    /** Color con el que se dibuja la funcin */
    private Color color;

    /** Nmero de puntos que se van a dibujar de la funcin. */
    private int numeroPuntos = 500;
}
