/**
 * Javier Abelln. 6 Abril 2006
 * 
 * Librera grfica
 */


package chuidiang.graficos.objetos_graficos;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import chuidiang.graficos.InterfaceEscalaGrafica;
import chuidiang.graficos.InterfaceZoomable;



/**
 * Pulsando el tercer botn del ratn y arrastrando, se puede mover el grfico.
 */
public class Mano
        implements ObservadorRaton
{
   //~ Variables de instancia --------------------------------------------------

   /** Lienzo de dibujo */
   private Component lienzo;

   /** Escala grfica sobre la que se va a arrastrar el grfico. */
   private InterfaceEscalaGrafica escala;

   /** Ultima posicion x,y del ratn, en coordenadas de usuario. */
   private double x;

   /** Ultima posicion x,y del ratn, en coordenadas de usuario. */
   private double y;

   //~ Constructores -----------------------------------------------------------

   /**
    * Construye una intancia de Mano.
    *
    * @param escala La escala sobre la que se debe arrastrar.
    * @param lienzo Lienzo de dibujo.<br>
    */
   public Mano( 
      InterfaceEscalaGrafica escala,
      Component lienzo )
   {
      this.escala = escala;
      this.lienzo = lienzo;
   }

   //~ Metodos -----------------------------------------------------------------

   /**
    * Mtodo al que se llama cuando se producen eventos de ratn. Modifica los
    * extremos de la escala grfica segn se mueve el ratn con el tercer
    * botn pulsado.
    *
    * @param e Evento del ratn.<br>
    * @param tipoEvento Tipo de evento del raton segn ObservadorRaton.<br>
    * @param x Valor x del evento de raton en coordenadas usuario.<br>
    * @param y Valor y del evento de raton en coordenadas usuario.<br>
    *
    * @return true si se ha producido arraste del ratn, para que se redibuje
    * el grfico.<br>
    */
   public boolean eventoRaton( 
      MouseEvent e,
      int tipoEvento,
      double x,
      double y )
   {
      // Si se pulsa el botn 3, se inicia todo y se pone un cursor de mano
      if( ( tipoEvento == ObservadorRaton.PULSADO ) &&
         ( e.getButton(  ) == MouseEvent.BUTTON3 ) )
      {
         this.x = e.getX(  );
         this.y = e.getY(  );
         this.lienzo.setCursor( new java.awt.Cursor( Cursor.HAND_CURSOR ) );

         return false;
      }

      // Mientras se arrastra, se mueve el grfico.
      if( ( tipoEvento == ObservadorRaton.ARRASTRE ) &&
         ( ( e.getModifiersEx(  ) & InputEvent.BUTTON3_DOWN_MASK ) == InputEvent.BUTTON3_DOWN_MASK ) )
      {
         Point2D origen =
            this.escala.dameCoordenadaUsuario( 
               new Point2D.Double( 
                  this.x,
                  this.y ) );
         Point2D destino =
            this.escala.dameCoordenadaUsuario( 
               new Point2D.Double( 
                  e.getX(  ),
                  e.getY(  ) ) );

         InterfaceZoomable zoom = this.escala.getZoomable(  );
         zoom.desplazar( 
            origen.getX(  ),
            origen.getY(  ),
            destino.getX(  ),
            destino.getY(  ) );
         this.x = e.getX(  );
         this.y = e.getY(  );

         return true;
      }

      // Al soltar se quita el cursor de mano.
      if( ( tipoEvento == ObservadorRaton.SOLTADO ) &&
         ( e.getButton(  ) == MouseEvent.BUTTON3 ) )
      {
         this.lienzo.setCursor( new java.awt.Cursor( Cursor.DEFAULT_CURSOR ) );

         return false;
      }

      return false;
   }
}
